import click
from rich import print as rich_print
from sifflet_sdk.constants import SIFFLET_CONFIG_CTX
from sifflet_sdk.status.service import StatusService

from sifflet_cli import __version__


@click.command()
@click.pass_context
def status(ctx):
    """
    Display basic status of sifflet cli
    """
    sifflet_config = ctx.obj[SIFFLET_CONFIG_CTX]

    show_status: str = f"""
Sifflet version = {__version__}
Tenant = {sifflet_config.backend_url or sifflet_config.tenant}"""

    rich_print(show_status)

    status_service = StatusService(sifflet_config)
    result: bool = status_service.check_status()
    if result:
        rich_print("[bold green]Status = OK[/bold green]")
    else:
        rich_print("[bold red]Status = KO[/bold red]")



from typing import Optional

import click
from click_option_group import RequiredMutuallyExclusiveOptionGroup, optgroup
from sifflet_sdk.config import SiffletConfig
from sifflet_sdk.configure.service import ConfigureService
from sifflet_sdk.constants import SIFFLET_CONFIG_CTX, TENANT_KEY, TOKEN_KEY


def default_from_context(default_key):
    class DefaultFromContext(click.Option):
        def get_default(self, ctx, call=True):
            sifflet_config: SiffletConfig = ctx.obj[SIFFLET_CONFIG_CTX]
            if default_key == TENANT_KEY:
                self.default = sifflet_config.tenant
            if default_key == TOKEN_KEY:
                self.default = sifflet_config.token
            return super().get_default(ctx)

        def __str__(self):
            print(default_key)
            return "*" * 8 if default_key == TOKEN_KEY else self.default

    return DefaultFromContext


@click.command()
@optgroup.group("Sifflet host configuration", cls=RequiredMutuallyExclusiveOptionGroup)
@optgroup.option(
    "--tenant",
    help="For SaaS version, name of your tenant. Sifflet UI URL is https://<tenant_name>.siffletdata.com. Sifflet "
    "Backend URL is https://<tenant_name>api.siffletdata.com. For instance, if you access to Sifflet UI with "
    "https://mycompany.siffletdata.com, then your tenant would be mycompany",
)
@optgroup.option(
    "--backend-url",
    help="For self-hosted deployment, full URL to the Sifflet backend on your deployment including for instance: "
    "https://sifflet-backend.mycompany.com",
)
@click.option(
    "--token",
    prompt="API access token",
    help="The access token must be generated in the Web UI of Sifflet Settings > Access-Tokens",
)
def configure(token: Optional[str] = None, tenant: Optional[str] = None, backend_url: Optional[str] = None):
    """Configure sifflet variables"""
    service = ConfigureService()
    service.configure(tenant, backend_url, token)



import click

from sifflet_cli.code.monitor.commands import monitor
from sifflet_cli.code.workspace.commands import workspace


@click.group()
def code():
    """Manage Sifflet resources as code."""


code.add_command(workspace)
code.add_command(monitor)



import logging

import click
from rich.logging import RichHandler


def get_logger() -> logging.Logger:
    """
    Returns a logger with the specified name.
    """
    logging.basicConfig(
        level=logging.INFO,
        format="%(message)s",
        datefmt="%X",
        handlers=[RichHandler(rich_tracebacks=True, tracebacks_suppress=[click], markup=True)],
    )

    return logging.getLogger("sifflet")


logger: logging.Logger = get_logger()



import sys

import click
from sifflet_sdk.configure.service import ConfigureService
from sifflet_sdk.constants import SIFFLET_CONFIG_CTX
from sifflet_sdk.errors import exception_handler

from sifflet_cli import __version__
from sifflet_cli.code.commands import code
from sifflet_cli.configure.commands import configure
from sifflet_cli.ingest.commands import ingest
from sifflet_cli.rules.commands import rules
from sifflet_cli.status.commands import status


@exception_handler
def main():
    """Entrypoint"""
    sys.exit(sifflet_cli())  # pylint: disable=E1120


@click.group()
@click.version_option(__version__)
@click.option("--debug", is_flag=True, hidden=True)
@click.pass_context
def sifflet_cli(ctx, debug: bool):
    """Sifflet CLI"""
    sifflet_config = ConfigureService().load_configuration(debug)
    ctx.ensure_object(dict)
    ctx.obj[SIFFLET_CONFIG_CTX] = sifflet_config


sifflet_cli.add_command(status)
sifflet_cli.add_command(configure)
sifflet_cli.add_command(rules)
sifflet_cli.add_command(ingest)
sifflet_cli.add_command(code)



from typing import List

import click
from click_aliases import ClickAliasedGroup
from sifflet_sdk.constants import (
    DEFAULT_PAGE_NUM,
    DEFAULT_PAGE_SIZE,
    OutputType,
    SIFFLET_CONFIG_CTX,
)
from sifflet_sdk.rules.service import RulesService

from sifflet_cli.rules.displayrules import DisplayRules


@click.group(cls=ClickAliasedGroup)
def rules():
    """List and control rules"""


@rules.command(name="list", aliases=["ls"])
@click.option("--name", "-n", type=str, required=False, help="Search rules by name")
@click.option(
    "--output",
    "-o",
    type=click.Choice(OutputType.list(), case_sensitive=False),
    required=False,
    help="Display the result either as a Table or raw Json",
    default="table",
    show_default=True,
)
@click.option(
    "--page-size",
    "page_size",
    type=int,
    required=False,
    help="Page size of the server side pagination",
    default=DEFAULT_PAGE_SIZE,
    show_default=True,
)
@click.option(
    "--page-num",
    "page_num",
    type=int,
    required=False,
    help="Page number of the server side pagination",
    default=DEFAULT_PAGE_NUM,
    show_default=True,
)
@click.pass_context
def list_rules(
    ctx,
    name: str,
    output: str = "table",
    page_size: int = DEFAULT_PAGE_SIZE,
    page_num: int = DEFAULT_PAGE_NUM,
):
    """Display all rules created"""
    sifflet_config = ctx.obj[SIFFLET_CONFIG_CTX]
    service = RulesService(sifflet_config, page_size=page_size, page_num=page_num)
    response_rules, response_total_count = service.fetch_rules(filter_name=name)
    utils = DisplayRules(output_type=output, page_num=page_num)
    utils.show_rules(rules=response_rules, total_count=response_total_count, filter_name=name)


@rules.command()
@click.option("--id", "ids", multiple=True, required=True, help="The rule id to trigger")
@click.pass_context
def run(ctx, ids: List[str]):
    """Run one or several rules - requires rule id(s)"""
    sifflet_config = ctx.obj[SIFFLET_CONFIG_CTX]
    service = RulesService(sifflet_config)
    rule_runs = service.run_rules(ids)
    service.wait_rule_runs(rule_runs)


@rules.command("run-history")
@click.option("--id", "rule_id", required=True, help="id of the rules id to fetch")
@click.option(
    "--output",
    "-o",
    type=click.Choice(OutputType.list(), case_sensitive=False),
    default="table",
    required=False,
    help="Display the result either as a Table or raw Json",
)
@click.option(
    "--page-size",
    "page_size",
    type=int,
    required=False,
    help="Page size of the server side pagination",
    default=DEFAULT_PAGE_SIZE,
    show_default=True,
)
@click.option(
    "--page-num",
    "page_num",
    type=int,
    required=False,
    help="Page number of the server side pagination",
    default=DEFAULT_PAGE_NUM,
    show_default=True,
)
@click.pass_context
def run_history(
    ctx,
    rule_id: str,
    output: str = "table",
    page_size: int = DEFAULT_PAGE_SIZE,
    page_num: int = DEFAULT_PAGE_NUM,
):
    """Display all rule runs for a given rule id"""
    sifflet_config = ctx.obj[SIFFLET_CONFIG_CTX]
    service = RulesService(sifflet_config, page_size=page_size, page_num=page_num)
    rule_info, rule_runs, total_count = service.fetch_run_history(rule_id=rule_id)
    utils = DisplayRules(output_type=output)
    utils.show_run_history(rule_info, rule_runs, total_count)



import json
from datetime import datetime
from typing import List, Optional

import rich
from rich import markup
from rich.console import Console
from rich.table import Table, Column

from sifflet_sdk.client.model.rule_catalog_asset_dto import RuleCatalogAssetDto
from sifflet_sdk.client.model.rule_info_dto import RuleInfoDto
from sifflet_sdk.client.model.rule_run_dto import RuleRunDto
from sifflet_cli.logger import logger
from sifflet_sdk.constants import OutputType, DEFAULT_PAGE_NUM, StatusError, StatusSuccess
from sifflet_sdk.errors import exception_handler


class DisplayRules:
    def __init__(self, output_type: str = OutputType.TABLE.value, page_num=DEFAULT_PAGE_NUM):
        self.console = Console()
        self.output_type = OutputType(output_type)
        self.page_num = page_num

    @exception_handler
    def show_rules(self, rules, total_count, filter_name):
        """Display rules in a table"""
        if rules:
            rules_cleaned = [
                {
                    "id": rule.get("id"),
                    "name": self._escape_markup(rule.get("name")),
                    "datasource_type": ", ".join([dataset.get("datasource_type") for dataset in rule.get("datasets")]),
                    "dataset_name": ", ".join([dataset.get("name") for dataset in rule.get("datasets")]),
                    "platform": rule.get("source_platform"),
                    "last_run_status": self._get_last_run_status(rule),
                    "last_run": self._get_last_run_timestamp(rule),
                }
                for rule in rules
            ]
            if self.output_type == OutputType.TABLE:
                table = Table()
                table.add_column("ID", no_wrap=True)
                table.add_column("Name", no_wrap=True)
                table.add_column("Datasource Type")
                table.add_column("Dataset")
                table.add_column("Platform")
                table.add_column("Last run status", justify="right")
                table.add_column("Last run date")
                for val in rules_cleaned:
                    table.add_row(*val.values())
                self.console.print(table)

                if len(rules) < int(total_count):
                    if self.page_num == 0:
                        self.console.print(f"Showing first {len(rules)} rules out of {total_count} rules")
                    else:
                        self.console.print(
                            f"Showing {len(rules)} rules out of {total_count} rules - page number {self.page_num}"
                        )
            else:
                self.console.print_json(json.dumps(rules_cleaned))
        elif filter_name:
            rich.print(f"No rule found for search filter: [bold]{filter_name}[/]")
        else:
            rich.print("No rule found")

    def _escape_markup(self, value: str):
        return markup.escape(value) if self.output_type == OutputType.TABLE else value

    def _format_status(self, status: str) -> str:
        result = status
        if self.output_type == OutputType.TABLE:
            if status in StatusError.__members__:
                result = f"[bold red]{status}[/bold red]"
            elif status in StatusSuccess.__members__:
                result = f"[bold green]{status}[/bold green]"

        return result

    def show_run_history(self, rule_info: RuleInfoDto, rule_runs: List[RuleRunDto], total_count: int):
        if rule_runs:
            rules_runs_cleaned = [
                {
                    "status": self._format_status(rule_run.get("status", default="")),
                    "start_date": str(datetime.fromtimestamp(rule_run.get("start_date", default=0) / 1000)),
                    "end_date": str(datetime.fromtimestamp(rule_run.get("end_date", default=0) / 1000)),
                    "type": self._escape_markup(rule_run.get("type")),
                    "result": self._escape_markup(rule_run.get("result")),
                }
                for rule_run in rule_runs
            ]
            table_title = (
                f"Rule name: [{rule_info.get('datasource_name')}]"
                f"[{rule_info.get('dataset_name')}]{rule_info.get('name')}"
            )
            if self.output_type == OutputType.TABLE:
                self.show_table(rules_runs_cleaned, title=table_title)
            else:
                self.console.print(rules_runs_cleaned)
            if len(rule_runs) < total_count:
                if self.page_num == 0:
                    self.console.print(f"Showing first {len(rule_runs)} runs out of {total_count} runs")
                else:
                    self.console.print(
                        f"Showing {len(rule_runs)} runs out of {total_count} runs - page number {self.page_num}"
                    )

    @staticmethod
    def _get_last_run_timestamp(rule: RuleCatalogAssetDto) -> str:
        if rule.get("last_run_status") and rule.get("last_run_status").timestamp:
            return str(datetime.fromtimestamp(rule.get("last_run_status").timestamp / 1000))
        else:
            return ""

    def _get_last_run_status(self, rule: RuleCatalogAssetDto) -> str:
        if rule.get("last_run_status") and rule.get("last_run_status").status:
            return self._format_status(rule.get("last_run_status").status)
        else:
            return ""

    @staticmethod
    def show_table(table: List[dict], title: Optional[str] = None) -> None:
        """Utility to display rich tables"""

        if table:
            header = list(table[0].keys())

            if "id" in header:
                header = list(filter(lambda x: x != "id", header))
                table_formatted = Table(Column(header="id", no_wrap=True), *header)
            else:
                table_formatted = Table(*header)

            for val in table:
                table_formatted.add_row(*val.values())

            table_formatted.title = title

            console = Console()
            console.print(table_formatted)
        else:
            logger.warning("No data to display")



import setuptools

import sifflet_cli

long_description = open("README.md", "r", encoding="utf-8").read()
requirements = [i.strip() for i in open("requirements.txt").readlines()]

setuptools.setup(
    name="sifflet",
    version=sifflet_cli.__version__,
    author="Sifflet",
    author_email="support@siffletdata.com",
    url="https://www.siffletdata.com/",
    description="Sifflet CLI",
    py_modules=["sifflet_cli"],
    entry_points={
        "console_scripts": [
            "sifflet = sifflet_cli.cli:main",
        ],
    },
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    packages=setuptools.find_packages(include=["sifflet_cli", "sifflet_cli.*"]),
    python_requires=">=3.7",
    install_requires=requirements,
)



from pathlib import Path

import click
from click import Context

from sifflet_cli.logger import logger
from sifflet_sdk.code.workspace.service import WorkspaceService
from sifflet_sdk.constants import SIFFLET_CONFIG_CTX


@click.group()
def monitor():
    """Manage monitors."""


@monitor.command()
@click.option("--file", "-f", "file_name", required=True, type=click.Path(), help="Path of the Monitor YAML file")
@click.option("--name", "-n", "name", required=True, type=str, help="Name of the monitor")
@click.pass_context
def init(ctx: Context, file_name: str, name: str):
    """
    Creates a new Monitor YAML file locally.
    """
    if Path(file_name).exists():
        raise click.BadParameter("File already exists", ctx, param_hint="--file")

    sifflet_config = ctx.obj[SIFFLET_CONFIG_CTX]
    workspace_service = WorkspaceService(sifflet_config)
    workspace_service.initialize_monitor(Path(file_name), name)
